package com.yuandian.dataflow;

 
import com.yuandian.dataflow.statemachine.RaftClosure;
import com.yuandian.dataflow.statemachine.StateMachine;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.SpringBootConfiguration;
import org.springframework.boot.autoconfigure.SpringBootApplication;

import java.io.File;
import java.nio.ByteBuffer;

import com.alipay.remoting.rpc.protocol.RpcResponseProcessor;
import com.alipay.sofa.jraft.Closure;
import com.alipay.sofa.jraft.Iterator;
import com.alipay.sofa.jraft.JRaftUtils;
import com.alipay.sofa.jraft.Node;
import com.alipay.sofa.jraft.NodeManager;
import com.alipay.sofa.jraft.RaftGroupService;
import com.alipay.sofa.jraft.RaftServiceFactory;
import com.alipay.sofa.jraft.conf.Configuration;
import com.alipay.sofa.jraft.core.IteratorImpl;
import com.alipay.sofa.jraft.core.NodeImpl;
import com.alipay.sofa.jraft.core.ReplicatorGroupImpl;
import com.alipay.sofa.jraft.entity.NodeId;
import com.alipay.sofa.jraft.entity.PeerId;
import com.alipay.sofa.jraft.entity.Task;
import com.alipay.sofa.jraft.option.NodeOptions;
import com.alipay.sofa.jraft.option.RaftOptions;
import com.alipay.sofa.jraft.rpc.CliClientService;
import com.alipay.sofa.jraft.rpc.RpcProcessor;
import com.alipay.sofa.jraft.rpc.RaftRpcServerFactory;
import com.alipay.sofa.jraft.rpc.RpcClient;
import com.alipay.sofa.jraft.rpc.RpcRequestProcessor;
import com.alipay.sofa.jraft.rpc.RpcServer;
import com.alipay.sofa.jraft.rpc.impl.BoltRpcServer;
import com.alipay.sofa.jraft.rpc.impl.BoltRaftRpcFactory;
import com.alipay.sofa.jraft.rpc.impl.BoltRpcClient;
import com.alipay.sofa.jraft.util.Endpoint;

import com.alipay.sofa.jraft.option.ReplicatorGroupOptions;

 


/**
 * Hello world!
 *
 */
@SpringBootApplication
public class Server {
    
    @Autowired
    public static Node node;
    public static RaftClosure done;
    
    public static Node GetNode()  {
        return node;
    }   

    public static RaftClosure GetDone()  {
        return done;
    }   
 

    public static void main(String[] args) {

        String[] peers = new String[]{"localhost:4440","localhost:4441","localhost:4442"};
        String[] sprPeers = new String[]{"3440","3441","3442"};


        var peeridstr = peers[ Integer.parseInt(args[0] ) ];
        var sprPort = sprPeers[Integer.parseInt(args[0] )];

        String groupId = "jraft";

        Configuration conf = JRaftUtils.getConfiguration("localhost:4440,localhost:4441,localhost:4442");

        PeerId serverId = JRaftUtils.getPeerId(peeridstr);
        int port = serverId.getPort();

        NodeOptions nodeOptions = new NodeOptions();
        nodeOptions.setElectionTimeoutMs(1000);
        nodeOptions.setSnapshotLogIndexMargin(3600);
        nodeOptions.setInitialConf(conf);

        File RaftDataFile = new File(String.format("./raftdata/%d", port) );
        System.out.print(RaftDataFile.mkdirs());  

        nodeOptions.setLogUri( String.format("./raftdata/%d/logs", port) );
        nodeOptions.setRaftMetaUri(String.format("./raftdata/%d/rafts", port));
        nodeOptions.setSnapshotUri(String.format("./raftdata/%d/snaps", port));
        nodeOptions.setFsm(new StateMachine());

        RaftGroupService cluster = new RaftGroupService(groupId, serverId, nodeOptions);
        node = cluster.start();
        done = new RaftClosure();

        System.setProperty("server.port", sprPort);
        var app = SpringApplication.run(Server.class, args);
        app.start();
 
        
        node.shutdown(done);
    }
}
