package com.yuandian.dataflow;

import java.time.Duration;
import java.time.Instant;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;
import java.util.Random;
import java.util.UUID;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.text.AbstractDocument.BranchElement;

import org.bson.Document;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.expression.spel.ast.FunctionReference;

import com.mongodb.MongoClient;
import com.mongodb.client.model.InsertManyOptions;
import com.yuandian.dataflow.projo.Doc;

import io.netty.handler.codec.dns.DatagramDnsQuery;
import lombok.Cleanup;
import lombok.Getter;
import lombok.Setter;
import lombok.extern.slf4j.Slf4j;

/**
 * Unit test for simple App.
 */
@DisplayName("AppTest")
@Slf4j
public class AppTest {

    @FunctionalInterface
    public interface FuncReturn {
        public float Execute();
    }

    public class Metric {

        public class ConditionExecute {
            
        }

        public class Element {
            public Duration Costtime;
            public float Value;

            public Element(Duration costtime, float value) {
                Costtime = costtime;
                Value = value;
            }
        }

        private LinkedBlockingQueue<Element> dataBlockQueue;
        private Queue<Element> countQueue;

        AtomicBoolean Running = new AtomicBoolean(true);

        public Metric() {
            dataBlockQueue = new LinkedBlockingQueue<Element>();
            countQueue = new LinkedList<Element>();
        }

        private Thread countWorker = new Thread(() -> {
            while (Running.get()) {

                try {
                    countQueue.add(dataBlockQueue.take());
                } catch (InterruptedException e1) {
                    e1.printStackTrace();
                    Runtime.getRuntime().exit(0);
                }

                if (countQueue.size() >= 24) {

                    Duration Cost = null;
                    float Total = 0;

                    var iter = countQueue.iterator();
                    while (iter.hasNext()) {
                        var e = iter.next();
                        if (Cost == null) {
                            Cost = e.Costtime;
                        } else {
                            Cost.plus(e.Costtime);
                        }
                        Total += e.Value;
                    }

                    log.info("Metric","qps: {}/s", (Total * 1000000000) / Cost.toNanos() );
                    countQueue.poll();
                }
            }
        });

        public void start() {
            if (!countWorker.isAlive()) {
                countWorker.start();
            }
        }

        public void close() {
            Running.set(false);
        }

        public void push(FuncReturn exec) {
            Instant now = Instant.now();
            var v = exec.Execute();
            dataBlockQueue.add(new Element(Duration.between(now, Instant.now()), v));
        }
    }

    @Test
    public void Mongodb() throws InterruptedException {

        ArrayList<Thread> execs = new ArrayList<>();

        final Metric metric = new Metric();
        metric.start();
        for (int c = 0; c < 10; c++) {
            Thread exec = new Thread(() -> {

                @Cleanup
                MongoClient mgo = new MongoClient("localhost", 27017);

                log.info("msg");

                long LoopNumber = 5;
                long BatchSize = 20000;

                var db = mgo.getDatabase("yuandian");
                var cltdoc = db.getCollection("doc");

                for (int n = 0; n < LoopNumber; n++) {

                    metric.push(() -> {

                        List<Doc> documents = new ArrayList<>();
                        Random r = new Random();

                        for (int i = 0; i < BatchSize; i++) {

                            var doc = new Doc();
                            var datadoc = new Document();

                            doc.append("code", r.nextInt(100));
                            doc.append("desc", "desc");
                            doc.append("ts", Instant.now());

                            for (int ii = 0; ii < 24; ii++) {
                                UUID uid = UUID.randomUUID();
                                datadoc
                                        .append(uid.toString(), uid.toString());
                            }

                            doc.append("data", datadoc);
                            documents.add(doc);
                        }

                        var opt = new InsertManyOptions();
                        cltdoc.insertMany(documents, opt);
                        return BatchSize;
                    });
                }
            });
            exec.start();
            execs.add(exec);
        }
        ;

        execs.forEach((e) -> {
            try {
                e.join();
            } catch (InterruptedException e1) {
                e1.printStackTrace();
            }
        });

        metric.close();

    }

}
